using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Reflection;
using Infragistics.Windows.DockManager;
using Infragistics.Windows.DockManager.Events;
using Infragistics.Windows.OutlookBar;
using Infragistics.Windows.Controls;
using System.Windows.Threading;

namespace BarclaysCapital.CR.PDC.Margin
{
    /// <summary>
    /// Interaction logic for ScreenSelector.xaml
    /// </summary>

    public partial class ScreenSelector : ScreenControl
    {
        private readonly List<ContentPane> _cpScreenList = new List<ContentPane>();

        public ScreenSelector()
        {
            InitializeComponent();
        }

        // Any additional screens to this group that contain a CounterpartySelector control should always call
        // ExecuteSelected() upon first selecting the OutlookBarGroup containing the CounterpartySelector control
        private void xamOutlookBar_SelectedGroupChanged(object sender, RoutedEventArgs e)
        {
            OutlookBarGroup obgSelected = ((XamOutlookBar)sender).SelectedGroup;

            if ((string)obgSelected.Header == ClientDashboard.Header)
            {
                if (!_cpScreenList.Exists(ScreenTypePredicate<ClientDashboard>))
                {
                    DisplayScreen<ClientDashboard>();
                }
            }
            else if ((string)obgSelected.Header == CallAndAlertManager.Header)
            {
                if (!_cpScreenList.Exists(ScreenTypePredicate<CallAndAlertManager>))
                {
                    ContentPane cp = DisplayScreen<CallAndAlertManager>();
                    Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)(() => ((CallAndAlertManager)cp.Content).SetData()));
                }
            }
        }

        private bool ScreenTypePredicate<T>(ContentPane cp)
        {
            return cp.Content is T;
        }

        private ContentPane DisplayScreen<T>() where T : ScreenControl, new()
        {
            // Create a local list of just the relevant screen types for working with
            List<ContentPane> screenList = _cpScreenList.FindAll(ScreenTypePredicate<T>);

            // Spawn a new window for the screen if option selected or if there are currently no windows
            if (screenList.Count == 0)
            {
                // Create a new pane for displaying the screen control
                ContentPane cp = CreateScreen<T>();
                screenList.Add(cp);
            }

            // Get the last screen in the list or the currently selected screen if it's of the correct type and a new window hasn't been spawned
            var cpSelected = (ContentPane)MainPage.Instance.tgpMain.SelectedItem;
            if (cpSelected == null || !(cpSelected.Content is T))
            {
                cpSelected = screenList[screenList.Count - 1];
                MakeScreenVisible(cpSelected);
            }

            return cpSelected;
        }

        private void MakeScreenVisible(ContentPane cpScreen)
        {
            ToolWindow tw = ToolWindow.GetToolWindow(cpScreen);

            if (tw != null)
            {
                tw.Activate();
            }
            else
            {
                cpScreen.Activate();
            }
        }

        private ContentPane CreateScreen<T>() where T : ScreenControl, new()
        {
            var screen = new T();

            // Create a new pane for displaying the screen control
            var cpScreen = new ContentPane { Content = screen };
            cpScreen.Closed += cpScreen_Closed;

            // Find the group relating to the template type (T) using the Header values and set the ContentPane properties accordingly
            foreach (OutlookBarGroup obgScreen in navigation.Groups)
            {
                if (obgScreen.Header == typeof(T).InvokeMember("Header", BindingFlags.GetProperty, null, null, null))
                {
                    cpScreen.Tag = obgScreen;
                    cpScreen.Header = obgScreen.Header;
                }
            }

            // Add the new content pane to the controlled member list (for tracking purposes) and as a new tab on the GUI
            _cpScreenList.Add(cpScreen);
            MainPage.Instance.tgpMain.Items.Add(cpScreen);

            screen.LoadLayout();
            return cpScreen;
        }

        private void cpScreen_Closed(object sender, PaneClosedEventArgs e)
        {
            // Only main screens (not panes within them) have been given this event handler, so check the 'Close' button
            // on the main screen ContentPane was clicked and not one of the panes within it
            if (e.Source == e.OriginalSource)
            {
                var cpScreen = (ContentPane)sender;
                ((ScreenControl)cpScreen.Content).SaveLayout();
                _cpScreenList.Remove(cpScreen);
            }
        }
    }

    public abstract class ScreenControl : UserControl
    {
        public virtual void SaveLayout()
        {
        }

        public virtual void LoadLayout()
        {
        }
    }
}